﻿
using iTextSharp.text;
using iTextSharp.text.pdf;
using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
//using System.Runtime.InteropServices;
/// <summary>
/// /// Aplicación de escritorio que dibuja las gráficas para la evaluación del paciente que 
/// use Physiobot, utilizando para ello consultas con linq.
/// Además damos las posibilidad de crear un informe con las métricas deseadas y los valores
/// más representativos.
/// ///////////////////////////////////////////////////////////////////////////////////////////////
/// Autor: Fernando del Pozo Ituero
/// ///////////////////////////////////////////////////////////////////////////////////////////////
/// Fecha: xx/xx/2019
/// </summary>
namespace Evolucion
{
    /// <summary>
    /// Formulario principal que registra los eventos y escoge los metodos a ejecutar.
    /// </summary>
    public partial class Formulario : Form
    {

        #region 
        /// <summary>
        /// variables utilizadas para el mpvimiento de la ventana del programa
        /// </summary>
        public int xClick = 0, yClick = 0;
        #endregion 
        /// <summary>
        /// Es la base de datos a la que le vamos a realizar consultas
        /// </summary>
         bbddDataContext bd = new bbddDataContext();
        /// <summary>
        /// Ruta en la que vamos a almacenar el PDF y temporalmente las imagenes que se le adjuntan
        /// </summary>
        public string Ruta = Application.StartupPath + "/"; 
        /// <summary>
        /// Es nuestro Formulario, en el cargamos la inicialización de los componentes
        /// </summary>
        public Formulario()
        {
            InitializeComponent();
        }
        /// <summary>
        /// Función que nos devuelve el valor de la terapia que vamos a utilizar.
        /// </summary>
        /// <returns>Devuelve un entero que represnta la terapa seleccionada</returns>
        protected int valorTipoTerapia()
        {            
            if (cbTipoTerapia.SelectedIndex < 0)
            {
                return -1;
            }
            else
            {
                if (cbTipoTerapia.SelectedIndex == 0) return 1;
                else if (cbTipoTerapia.SelectedIndex == 1) return 2;
                else if (cbTipoTerapia.SelectedIndex == 2) return 3;
                else if (cbTipoTerapia.SelectedIndex == 3) return 4;
                else if (cbTipoTerapia.SelectedIndex == 4) return 5;
                else if (cbTipoTerapia.SelectedIndex == 5) return 6;
                else if (cbTipoTerapia.SelectedIndex == 6) return 7;
                else if (cbTipoTerapia.SelectedIndex == 7) return 8;
                else if (cbTipoTerapia.SelectedIndex == 8) return 9;
                else if (cbTipoTerapia.SelectedIndex == 9) return 10;
                else return -1;
            }
        }
        /// <summary>
        /// Funcion que se encarga de ejecutar la función que va a generar el grafico en el programa.
        /// </summary>
        protected void cargarSeleccionGafico()
        {
            if (Metodos.comprobarSeleccion(cbPaciente, cbTipoTerapia) && CbRangoTerapiaSuperior.SelectedIndex > cbRangoTerapiaInferior.SelectedIndex)
            {
                int seleccion = cbTipoGrafico.SelectedIndex;
                Document vacio = new Document();
                if (seleccion == 0)
                {
                    Metodos.graficaMarcador(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 1)
                {
                    Metodos.graficaError(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 2)
                {
                    Metodos.graficaMedia(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 3)
                {
                    Metodos.graficaMediaVelocidad(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 4)
                {
                    Metodos.graficaTiempoReacion(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 5)
                {
                    Metodos.graficaNumeroDeTemblores(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 6)
                {
                    Metodos.graficaDireccional(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 7)
                {
                    Metodos.graficaSimetriaVA(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else if (seleccion == 8)
                {
                    Metodos.graficaEvolucionFuerza(false, vacio, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                }
                else
                {
                    MessageBox.Show("Seleccione el tipo de grafico");
                }
            }
            else // Comprobamos si el error es porque el rango no es lógico
            {
                if (CbRangoTerapiaSuperior.SelectedIndex <= cbRangoTerapiaInferior.SelectedIndex)
                {
                    MessageBox.Show("Porfavor, Elija un rango de tiempo coherente");
                }
            }
        }
        /// <summary>
        /// Esta función es la encargade de generar el informe, tanto como la elaboración de una portada como las métricas que se van a gráficar
        /// </summary>
        protected void generarInforme()
        {      
            if (Metodos.comprobarSeleccion(cbPaciente,cbTipoTerapia) && CbRangoTerapiaSuperior.SelectedIndex > cbRangoTerapiaInferior.SelectedIndex)
            {
                //Generación de la portada del informe
                Document Informe = new Document();
                PdfWriter.GetInstance(Informe, new FileStream("Evolución_del_paciente" + cbPaciente.SelectedValue + ".pdf", FileMode.OpenOrCreate));
                Informe.Open();
                Chunk Titulo = new Chunk(" EVOLUCIÓN DEL PACIENTE " + cbPaciente.SelectedValue, FontFactory.GetFont("ARIAL", 24, iTextSharp.text.Font.UNDERLINE, BaseColor.DARK_GRAY));
                Paragraph Portada = new Paragraph(Titulo);
                Portada.Alignment = Element.ALIGN_CENTER;
                Informe.Add(new Paragraph("\n\n\n\n\n\n"));
                Informe.Add(Portada);
                Informe.Add(new Paragraph("\n\n\n"));
                iTextSharp.text.Image PHYSIOBOT = iTextSharp.text.Image.GetInstance(Ruta + "/LOGO_PHYSIOBIOBOT.PNG");
                iTextSharp.text.Image cartif = iTextSharp.text.Image.GetInstance(Ruta + "/Logo_Cartif.PNG");
                cartif.Alignment = Element.ALIGN_MIDDLE;
                PHYSIOBOT.Alignment = Element.ALIGN_MIDDLE;
                cartif.ScalePercent((275 / cartif.Width) * 100);//Dimensionamiento del logo de CARTIF
                PHYSIOBOT.ScalePercent((225 / cartif.Width) * 100);//Dimensionamiento del logo de E2ROBOT
                Informe.Add(new Paragraph("\n\n\n"));
                Informe.Add(cartif);
                Informe.Add(PHYSIOBOT);
                Informe.Add(new Paragraph("\n\n Fecha del Informe: " + DateTime.Now.ToString()));
                Informe.Add(new Paragraph("\n\n El tipo de terapia es: " + valorTipoTerapia()));

                Informe.AddCreationDate();

                if (ListaGraficos.GetItemChecked(0))
                {
                    //Seleccionamos las métricas que han sido seleccionadas por el paciente y les mandamos el informe para que gráfiquen y nos den los datos que necesitamos.
                    //if (valorTipoTerapia() == 3)
                    //{
                        Informe.NewPage();
                        Informe.NewPage();
                        Informe.Add(PHYSIOBOT);
                        Metodos.generaCabecero(Informe, " Marcador", cbPaciente, valorTipoTerapia());
                        Metodos.graficaMarcador(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                        ProgresoInforme.Value = +10;
                    //}
                }
                if (ListaGraficos.GetItemChecked(1))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Porcentaje de error", cbPaciente, valorTipoTerapia());
                    Metodos.graficaError(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                }
                if (ListaGraficos.GetItemChecked(2))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Media y desviación",cbPaciente, valorTipoTerapia());
                    Metodos.graficaMedia(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                }
                if (ListaGraficos.GetItemChecked(3))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Facilidad de movimiento", cbPaciente, valorTipoTerapia());
                    Metodos.graficaMediaVelocidad(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                }
                if (ListaGraficos.GetItemChecked(4))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Tiempo de reacción", cbPaciente, valorTipoTerapia());
                    Metodos.graficaTiempoReacion(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                    ProgresoInforme.Update();
                }                
                if (ListaGraficos.GetItemChecked(5))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Número de temblores", cbPaciente, valorTipoTerapia());
                    Metodos.graficaNumeroDeTemblores(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                }
                if (ListaGraficos.GetItemChecked(6))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Simetría direccional x e y", cbPaciente, valorTipoTerapia());
                    Metodos.graficaDireccional(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                }
                if (ListaGraficos.GetItemChecked(7))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Simetría velocidad y ángulo", cbPaciente, valorTipoTerapia());
                    Metodos.graficaSimetriaVA(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                    ProgresoInforme.Update();
                }
                if (ListaGraficos.GetItemChecked(8))
                {
                    Informe.NewPage();
                    Informe.NewPage();
                    Informe.Add(PHYSIOBOT);
                    Metodos.generaCabecero(Informe, " Evolución de la fuerza", cbPaciente, valorTipoTerapia());                    
                    Metodos.graficaEvolucionFuerza(true, Informe, valorTipoTerapia(), cbPaciente, CbRangoTerapiaSuperior, cbRangoTerapiaInferior, GrafTrayectoria, bd, Ruta);
                    ProgresoInforme.Value = +10;
                }
                ProgresoInforme.Value = 100;
                Informe.Close();

            }
            else // Comprobamos si el error es porque el rango no es lógico
            {
                if (CbRangoTerapiaSuperior.SelectedIndex <= cbRangoTerapiaInferior.SelectedIndex)
                {
                    MessageBox.Show("Porfavor, Elija un rango de tiempo coherente");
                }
            }
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            Fondo.Visible = true;
            GrafTrayectoria.Visible = false;
            pictureCartif.Visible = false;
            ProgresoInforme.Visible = false;
            CargaDatos.cargarCbTipoTerapia(cbTipoTerapia);           
            CargaDatos.cargarCbPacientes(cbPaciente, bd);
            CargaDatos.cargarCbGraficas(cbTipoGrafico);
            CargaDatos.cargarListaCheckBox(ListaGraficos);
            CargaDatos.desactivarControles(cbTipoTerapia, cbTipoGrafico, BotonCargarGrafica, cbRangoTerapiaInferior, CbRangoTerapiaSuperior, ListaGraficos, botonGenerarInforme);
        }

        private void cbPaciente_SelectionChangeCommitted(object sender, EventArgs e)
        {
            cbTipoTerapia.Enabled = true;
            CargaDatos.cargarCbRango(CbRangoTerapiaSuperior, cbRangoTerapiaInferior, bd, cbPaciente);
        }
        private void cbTipoTerapia_SelectionChangeCommitted(object sender, EventArgs e)
        {
            cbTipoGrafico.Enabled = true;
            cbRangoTerapiaInferior.Enabled = true;
            CbRangoTerapiaSuperior.Enabled = true;
            BotonCargarGrafica.Enabled = true;
            ListaGraficos.Enabled = true;
            botonGenerarInforme.Enabled = true;
        }

        private void BotonCargarGrafica_Click_1(object sender, EventArgs e)
        {
            Fondo.Visible = false;
            GrafTrayectoria.Visible = true;
            pictureCartif.Visible = true;
            cargarSeleccionGafico();
        }
        private void cbTipoGrafico_SelectionChangeCommitted(object sender, EventArgs e)
        {
            BotonCargarGrafica.Enabled = true;
        }


        private void botonGenerarInforme_Click_1(object sender, EventArgs e)
        {
            ProgresoInforme.Value = 0;
            ProgresoInforme.Visible = true;
            Fondo.Visible = true;
            GrafTrayectoria.Visible = false;
            pictureCartif.Visible = false;
            generarInforme();
            ProgresoInforme.Visible = false;
            Process proceso = new Process();
            proceso.StartInfo.FileName = Ruta + ("Evolución_del_paciente" + cbPaciente.SelectedValue + ".pdf");
            proceso.Start();
            proceso.Close();
        }

        #region
        /// Para mejorar la estética hemos creado nuevos botones así como efecttos.
        /// Las funciones y eventos que tenemos a partir de aqui solo aportan estética al programa
        /// y no realizan ningun tratamiento de datos.

        private void BoxMinimizar_Click(object sender, EventArgs e)
        {
            WindowState = FormWindowState.Minimized;
        }


        private void BoxCerrar_Click_1(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void BoxMaximizar_Click_1(object sender, EventArgs e)
        {
            WindowState = FormWindowState.Maximized;
            BoxMaximizar.Visible = false;
            BoxRestaurar.Visible = true;
        }

        private void BoxRestaurar_Click_1(object sender, EventArgs e)
        {
            WindowState = FormWindowState.Normal;
            BoxMaximizar.Visible = true;
            BoxRestaurar.Visible = false;
        }

        private void panel3_MouseMove(object sender, MouseEventArgs e)
        {
            // Para realizar el movimiento del panel.
            if (e.Button != MouseButtons.Left)
            { xClick = e.X; yClick = e.Y; }
            else
            { this.Left = this.Left + (e.X - xClick); this.Top = this.Top + (e.Y - yClick); }
        }

        private void BoxMinimizar_Click_1(object sender, EventArgs e)
        {

                WindowState = FormWindowState.Minimized;

        }
        #endregion

    }
}